//////////////////////////////////////////////////////////////////////////////
//
// PocketHAL
// Copyright 2003 by Thierry Tremblay
//
//////////////////////////////////////////////////////////////////////////////

#ifndef POCKETHAL_DISPLAY_H
#define POCKETHAL_DISPLAY_H

#include <windows.h>
#include "stdint.h"
#include "utility.h"

namespace PHAL
{

class Surface;



//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////

enum Orientation
{
    // Orientations
    ORIENTATION_NORMAL,         // ex: Casio E-125
    ORIENTATION_ROTATE90CCW,    // ex: iPAQ 3600
    ORIENTATION_ROTATE180,      // ??? (Upside down)
    ORIENTATION_ROTATE90CW,     // ex: iPAQ 3800

    // Special
    ORIENTATION_NATIVE,         // Use the device's native orientation

    // Aliases
    ORIENTATION_PORTRAIT    = ORIENTATION_NORMAL,
    ORIENTATION_LANDSCAPE   = ORIENTATION_ROTATE90CCW,
    ORIENTATION_FASTEST     = ORIENTATION_NATIVE
};



enum SwapEffect
{
    SWAP_NONE,      // Direct access (no buffering)
    SWAP_DISCARD,   // We don't care about the content of the backbuffer
    SWAP_COPY,      // We want to keep the backbuffer content
//    SWAP_FLIP,      // Alternate between two buffers
};



//////////////////////////////////////////////////////////////////////////////
//
// DisplayParameters
//
//////////////////////////////////////////////////////////////////////////////

struct DisplayParameters
{
    Orientation m_orientation;      // Orientation of screen
    SwapEffect  m_swapEffect;       // Swap effect
    bool        m_swapVSYNC;        // Swap on VSYNC

    Orientation m_inputOrientation; // Orientation for stylus/arrow keys

    DisplayParameters()
        :   m_orientation( ORIENTATION_NORMAL ),
            m_swapEffect( SWAP_DISCARD ),
            m_swapVSYNC( false )
        {}
};



//////////////////////////////////////////////////////////////////////////////
//
// Display
//
//////////////////////////////////////////////////////////////////////////////

class Display : NonCopyable
{
public:

    static Display* Create();
    
    virtual bool Initialize( HWND hwnd, DisplayParameters parameters = DisplayParameters() ) = 0;
    virtual void Shutdown() = 0;


    // Return the backbuffer. The non-const version will force a locking of the backbuffer.
    virtual const Surface* GetBackBuffer() const = 0;
    virtual Surface*       GetBackBuffer() = 0;

    // Return the orientation of the display
    virtual Orientation GetOrientation() const = 0;

    // Set the update viewport
    virtual void SetViewport( const RECT* viewport ) = 0;

    // Update the display with the content of the backbuffer (i.e. flip).
    virtual void Swap() = 0;


    // Implementation details
    virtual ~Display() {}


protected:

    Display() {}
};



} // end of namespace PHAL

#endif
