//////////////////////////////////////////////////////////////////////////////
//
// PocketHAL
// Copyright 2003 by Thierry Tremblay
//
//////////////////////////////////////////////////////////////////////////////

#ifndef POCKETHAL_SURFACE_H
#define POCKETHAL_SURFACE_H

#include "stdint.h"
#include "utility.h"

namespace PHAL
{


//////////////////////////////////////////////////////////////////////////////
//
// Surface
//
//////////////////////////////////////////////////////////////////////////////

class Surface : NonCopyable
{
public:

    // Create a surface and allocate the needed memory. The surface will be
    // padded to ensure lines are 32-bits aligned.
    Surface( unsigned width, unsigned height );

    // Create a surface from an existing memory location. All the surface
    // parameters must be specified.
    Surface( uint16_t* pixels, unsigned width, unsigned height, unsigned pitch );
    
    // Destroy the surface, freeing any allocated memory
    virtual ~Surface();

    // Accessors
    unsigned        GetWidth() const                 { return m_width; }
    unsigned        GetHeight() const                { return m_height; }
    unsigned        GetPitch() const                 { return m_pitch; }

    uint16_t*       GetPixels()                      { return m_pixels; }
    const uint16_t* GetPixels() const                { return m_pixels; }
    uint16_t*       GetPixels( int x, int y )        { return (uint16_t*)((char*)(m_pixels + x) + y * m_pitch); }
    const uint16_t* GetPixels( int x, int y ) const  { return (uint16_t*)((char*)(m_pixels + x) + y * m_pitch); }


protected:

    unsigned  m_width;
    unsigned  m_height;
    unsigned  m_pitch;
    uint16_t* m_pixels;
    bool      m_bOwnMemory;
};



} // end of namespace PHAL

#endif
